@@ -1,37 +1,14 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 #include "protobuf.h"
 
-#include <php.h>
 #include <Zend/zend_interfaces.h>
+#include <php.h>
 
 #include "arena.h"
 #include "array.h"
@@ -52,21 +29,19 @@
 
 #define PROTOBUF_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(protobuf, v)
 
+// clang-format off
 ZEND_BEGIN_MODULE_GLOBALS(protobuf)
   // Set by the user to make the descriptor pool persist between requests.
   zend_bool keep_descriptor_pool_after_request;
 
-  // Currently we make the generated pool a "global", which means that if a user
-  // does explicitly create threads within their request, the other threads will
-  // get different results from DescriptorPool::getGeneratedPool(). We require
-  // that all descriptors are loaded from the main thread.
-  zval generated_pool;
+  // Set by the user to make the descriptor pool persist between requests.
+  zend_class_entry* constructing_class;
 
   // A upb_DefPool that we are saving for the next request so that we don't have
   // to rebuild it from scratch. When keep_descriptor_pool_after_request==true,
   // we steal the upb_DefPool from the global DescriptorPool object just before
   // destroying it.
-  upb_DefPool *global_symtab;
+  upb_DefPool* global_symtab;
 
   // Object cache (see interface in protobuf.h).
   HashTable object_cache;
@@ -81,8 +56,9 @@ ZEND_BEGIN_MODULE_GLOBALS(protobuf)
   // destroyed on request shutdown.
   HashTable descriptors;
 ZEND_END_MODULE_GLOBALS(protobuf)
+// clang-format on
 
-void free_protobuf_globals(zend_protobuf_globals *globals) {
+void free_protobuf_globals(zend_protobuf_globals* globals) {
   zend_hash_destroy(&globals->name_msg_cache);
   zend_hash_destroy(&globals->name_enum_cache);
   upb_DefPool_Free(globals->global_symtab);
@@ -91,9 +67,7 @@ void free_protobuf_globals(zend_protobuf_globals *globals) {
 
 ZEND_DECLARE_MODULE_GLOBALS(protobuf)
 
-const zval *get_generated_pool() {
-  return &PROTOBUF_G(generated_pool);
-}
+upb_DefPool* get_global_symtab() { return PROTOBUF_G(global_symtab); }
 
 // This is a PHP extension (not a Zend extension). What follows is a summary of
 // a PHP extension's lifetime and when various handlers are called.
@@ -147,7 +121,8 @@ const zval *get_generated_pool() {
 //     * The GitHub master branch supports 7.2+, but this has not been released
 //       to PECL.
 //     * Its owner has disavowed it as "broken by design" and "in an untenable
-//       position for the future": https://github.com/krakjoe/pthreads/issues/929
+//       position for the future":
+//       https://github.com/krakjoe/pthreads/issues/929
 //   * The only way to use PHP with requests in different threads is to use the
 //     Apache 2 mod_php with the "worker" MPM. But this is explicitly
 //     discouraged by the documentation: https://serverfault.com/a/231660
@@ -158,10 +133,7 @@ static PHP_GSHUTDOWN_FUNCTION(protobuf) {
   }
 }
 
-static PHP_GINIT_FUNCTION(protobuf) {
-  ZVAL_NULL(&protobuf_globals->generated_pool);
-  protobuf_globals->global_symtab = NULL;
-}
+static PHP_GINIT_FUNCTION(protobuf) { protobuf_globals->global_symtab = NULL; }
 
 /**
  * PHP_RINIT_FUNCTION(protobuf)
@@ -171,16 +143,16 @@ static PHP_GINIT_FUNCTION(protobuf) {
 static PHP_RINIT_FUNCTION(protobuf) {
   // Create the global generated pool.
   // Reuse the symtab (if any) left to us by the last request.
-  upb_DefPool *symtab = PROTOBUF_G(global_symtab);
-  if (!symtab) {
-    PROTOBUF_G(global_symtab) = symtab = upb_DefPool_New();
-    zend_hash_init(&PROTOBUF_G(name_msg_cache), 64, NULL, NULL, 0);
-    zend_hash_init(&PROTOBUF_G(name_enum_cache), 64, NULL, NULL, 0);
+  if (!PROTOBUF_G(global_symtab)) {
+    zend_bool persistent = PROTOBUF_G(keep_descriptor_pool_after_request);
+    PROTOBUF_G(global_symtab) = upb_DefPool_New();
+    zend_hash_init(&PROTOBUF_G(name_msg_cache), 64, NULL, NULL, persistent);
+    zend_hash_init(&PROTOBUF_G(name_enum_cache), 64, NULL, NULL, persistent);
   }
-  DescriptorPool_CreateWithSymbolTable(&PROTOBUF_G(generated_pool), symtab);
 
   zend_hash_init(&PROTOBUF_G(object_cache), 64, NULL, NULL, 0);
   zend_hash_init(&PROTOBUF_G(descriptors), 64, NULL, ZVAL_PTR_DTOR, 0);
+  PROTOBUF_G(constructing_class) = NULL;
 
   return SUCCESS;
 }
@@ -196,7 +168,6 @@ static PHP_RSHUTDOWN_FUNCTION(protobuf) {
     free_protobuf_globals(ZEND_MODULE_GLOBALS_BULK(protobuf));
   }
 
-  zval_dtor(&PROTOBUF_G(generated_pool));
   zend_hash_destroy(&PROTOBUF_G(object_cache));
   zend_hash_destroy(&PROTOBUF_G(descriptors));
 
@@ -207,7 +178,7 @@ static PHP_RSHUTDOWN_FUNCTION(protobuf) {
 // Object Cache.
 // -----------------------------------------------------------------------------
 
-void Descriptors_Add(zend_object *desc) {
+void Descriptors_Add(zend_object* desc) {
   // The hash table will own a ref (it will destroy it when the table is
   // destroyed), but for some reason the insert operation does not add a ref, so
   // we do that here with ZVAL_OBJ_COPY().
@@ -216,12 +187,12 @@ void Descriptors_Add(zend_object *desc) {
   zend_hash_next_index_insert(&PROTOBUF_G(descriptors), &zv);
 }
 
-void ObjCache_Add(const void *upb_obj, zend_object *php_obj) {
+void ObjCache_Add(const void* upb_obj, zend_object* php_obj) {
   zend_ulong k = (zend_ulong)upb_obj;
   zend_hash_index_add_ptr(&PROTOBUF_G(object_cache), k, php_obj);
 }
 
-void ObjCache_Delete(const void *upb_obj) {
+void ObjCache_Delete(const void* upb_obj) {
   if (upb_obj) {
     zend_ulong k = (zend_ulong)upb_obj;
     int ret = zend_hash_index_del(&PROTOBUF_G(object_cache), k);
@@ -229,9 +200,9 @@ void ObjCache_Delete(const void *upb_obj) {
   }
 }
 
-bool ObjCache_Get(const void *upb_obj, zval *val) {
+bool ObjCache_Get(const void* upb_obj, zval* val) {
   zend_ulong k = (zend_ulong)upb_obj;
-  zend_object *obj = zend_hash_index_find_ptr(&PROTOBUF_G(object_cache), k);
+  zend_object* obj = zend_hash_index_find_ptr(&PROTOBUF_G(object_cache), k);
 
   if (obj) {
     ZVAL_OBJ_COPY(val, obj);
@@ -246,35 +217,35 @@ bool ObjCache_Get(const void *upb_obj, zval *val) {
 // Name Cache.
 // -----------------------------------------------------------------------------
 
-void NameMap_AddMessage(const upb_MessageDef *m) {
-  char *k = GetPhpClassname(upb_MessageDef_File(m), upb_MessageDef_FullName(m));
-  zend_hash_str_add_ptr(&PROTOBUF_G(name_msg_cache), k, strlen(k), (void*)m);
-  free(k);
+void NameMap_AddMessage(const upb_MessageDef* m) {
+  for (int i = 0; i < 2; ++i) {
+    char* k = GetPhpClassname(upb_MessageDef_File(m),
+                              upb_MessageDef_FullName(m), (bool)i);
+    zend_hash_str_add_ptr(&PROTOBUF_G(name_msg_cache), k, strlen(k), (void*)m);
+    if (!IsPreviouslyUnreservedClassName(k)) {
+      free(k);
+      return;
+    }
+    free(k);
+  }
 }
 
-void NameMap_AddEnum(const upb_EnumDef *e) {
-  char *k = GetPhpClassname(upb_EnumDef_File(e), upb_EnumDef_FullName(e));
+void NameMap_AddEnum(const upb_EnumDef* e) {
+  char* k =
+      GetPhpClassname(upb_EnumDef_File(e), upb_EnumDef_FullName(e), false);
   zend_hash_str_add_ptr(&PROTOBUF_G(name_enum_cache), k, strlen(k), (void*)e);
   free(k);
 }
 
-const upb_MessageDef *NameMap_GetMessage(zend_class_entry *ce) {
-  const upb_MessageDef *ret =
+const upb_MessageDef* NameMap_GetMessage(zend_class_entry* ce) {
+  const upb_MessageDef* ret =
       zend_hash_find_ptr(&PROTOBUF_G(name_msg_cache), ce->name);
 
-  if (!ret && ce->create_object) {
-#if PHP_VERSION_ID < 80000
-    zval tmp;
+  if (!ret && ce->create_object && ce != PROTOBUF_G(constructing_class)) {
     zval zv;
-    ZVAL_OBJ(&tmp, ce->create_object(ce));
-    zend_call_method_with_0_params(&tmp, ce, NULL, "__construct", &zv);
-    zval_ptr_dtor(&tmp);
-#else
-    zval zv;
-    zend_object *tmp = ce->create_object(ce);
+    zend_object* tmp = ce->create_object(ce);
     zend_call_method_with_0_params(tmp, ce, NULL, "__construct", &zv);
     OBJ_RELEASE(tmp);
-#endif
     zval_ptr_dtor(&zv);
     ret = zend_hash_find_ptr(&PROTOBUF_G(name_msg_cache), ce->name);
   }
@@ -282,30 +253,35 @@ const upb_MessageDef *NameMap_GetMessage(zend_class_entry *ce) {
   return ret;
 }
 
-const upb_EnumDef *NameMap_GetEnum(zend_class_entry *ce) {
-  const upb_EnumDef *ret =
+const upb_EnumDef* NameMap_GetEnum(zend_class_entry* ce) {
+  const upb_EnumDef* ret =
       zend_hash_find_ptr(&PROTOBUF_G(name_enum_cache), ce->name);
   return ret;
 }
 
+void NameMap_EnterConstructor(zend_class_entry* ce) {
+  assert(!PROTOBUF_G(constructing_class));
+  PROTOBUF_G(constructing_class) = ce;
+}
+
+void NameMap_ExitConstructor(zend_class_entry* ce) {
+  assert(PROTOBUF_G(constructing_class) == ce);
+  PROTOBUF_G(constructing_class) = NULL;
+}
+
 // -----------------------------------------------------------------------------
 // Module init.
 // -----------------------------------------------------------------------------
 
-zend_function_entry protobuf_functions[] = {
-  ZEND_FE_END
-};
+zend_function_entry protobuf_functions[] = {ZEND_FE_END};
 
-static const zend_module_dep protobuf_deps[] = {
-  ZEND_MOD_OPTIONAL("date")
-  ZEND_MOD_END
-};
+static const zend_module_dep protobuf_deps[] = {ZEND_MOD_OPTIONAL("date")
+                                                    ZEND_MOD_END};
 
 PHP_INI_BEGIN()
 STD_PHP_INI_ENTRY("protobuf.keep_descriptor_pool_after_request", "0",
-                  PHP_INI_ALL, OnUpdateBool,
-                  keep_descriptor_pool_after_request, zend_protobuf_globals,
-                  protobuf_globals)
+                  PHP_INI_ALL, OnUpdateBool, keep_descriptor_pool_after_request,
+                  zend_protobuf_globals, protobuf_globals)
 PHP_INI_END()
 
 static PHP_MINIT_FUNCTION(protobuf) {
@@ -325,22 +301,21 @@ static PHP_MSHUTDOWN_FUNCTION(protobuf) {
 }
 
 zend_module_entry protobuf_module_entry = {
-  STANDARD_MODULE_HEADER_EX,
-  NULL,
-  protobuf_deps,
-  "protobuf",               // extension name
-  protobuf_functions,       // function list
-  PHP_MINIT(protobuf),      // process startup
-  PHP_MSHUTDOWN(protobuf),  // process shutdown
-  PHP_RINIT(protobuf),      // request shutdown
-  PHP_RSHUTDOWN(protobuf),  // request shutdown
-  NULL,                     // extension info
-  PHP_PROTOBUF_VERSION,     // extension version
-  PHP_MODULE_GLOBALS(protobuf),  // globals descriptor
-  PHP_GINIT(protobuf),      // globals ctor
-  PHP_GSHUTDOWN(protobuf),  // globals dtor
-  NULL,                     // post deactivate
-  STANDARD_MODULE_PROPERTIES_EX
-};
+    STANDARD_MODULE_HEADER_EX,
+    NULL,
+    protobuf_deps,
+    "protobuf",                    // extension name
+    protobuf_functions,            // function list
+    PHP_MINIT(protobuf),           // process startup
+    PHP_MSHUTDOWN(protobuf),       // process shutdown
+    PHP_RINIT(protobuf),           // request startup
+    PHP_RSHUTDOWN(protobuf),       // request shutdown
+    NULL,                          // extension info
+    PHP_PROTOBUF_VERSION,          // extension version
+    PHP_MODULE_GLOBALS(protobuf),  // globals descriptor
+    PHP_GINIT(protobuf),           // globals ctor
+    PHP_GSHUTDOWN(protobuf),       // globals dtor
+    NULL,                          // post deactivate
+    STANDARD_MODULE_PROPERTIES_EX};
 
 ZEND_GET_MODULE(protobuf)
