@@ -1111,7 +1111,7 @@ class JsonFormatTest(JsonFormatBase):
     json_format.Parse(text, message)
     self.assertEqual(message.bytes_value, b'\x01\x02')
 
-  def testParseBadIdentifer(self):
+  def testParseBadIdentifier(self):
     self.CheckError(
         '{int32Value: 1}',
         (
@@ -1668,6 +1668,11 @@ class JsonFormatTest(JsonFormatBase):
         "Failed to parse JSON: TypeError: 'int' object is not iterable.",
     )
 
+  def testManyRecursionsRaisesParseError(self):
+    num_recursions = 1050
+    text = ('{"a":' * num_recursions) + '""' + ('}' * num_recursions)
+    with self.assertRaises(json_format.ParseError):
+      json_format.Parse(text, json_format_proto3_pb2.TestMessage())
 
 if __name__ == '__main__':
   unittest.main()
