@@ -117,6 +117,9 @@ TEST(FileTest, TopologicallyOrderedDescriptors) {
       "OneBytes",
       "MoreString",
       "MoreBytes",
+      "MessageCreatorZeroInit",
+      "MessageCreatorMemcpy.M2Entry",
+      "MessageCreatorFunc",
       "ManyOptionalString",
       "Int64ParseTester",
       "Int64Message",
@@ -155,6 +158,7 @@ TEST(FileTest, TopologicallyOrderedDescriptors) {
       "TestCamelCaseFieldNames",
       "TestAllTypes",
       "RedactedFields",
+      "MessageCreatorMemcpy",
       "TestVerifyUint32BigFieldNumber",
       "TestVerifyUint32",
       "TestVerifyOneUint32",
@@ -208,9 +212,6 @@ TEST(FileTest, TopologicallyOrderedDescriptors) {
     EXPECT_TRUE(match) << "failed to match; expected "
                        << kExpectedDescriptorOrder[i] << ", got "
                        << desc->full_name();
-    if (!match) {
-      break;
-    }
   }
 }
 
