@@ -175,6 +175,8 @@ bool RustGenerator::Generate(const FileDescriptor* file,
       {"pbr", "::__pb::__runtime"},
       {"NonNull", "::__std::ptr::NonNull"},
       {"Phantom", "::__std::marker::PhantomData"},
+      {"Result", "::__std::result::Result"},
+      {"Option", "::__std::option::Option"},
   });
 
   ctx.Emit({{"kernel", KernelRsName(ctx.opts().kernel)}}, R"rs(
@@ -205,6 +207,11 @@ bool RustGenerator::Generate(const FileDescriptor* file,
          {"proto_deps_h",
           [&] {
             for (int i = 0; i < file->dependency_count(); i++) {
+              if (opts->strip_nonfunctional_codegen &&
+                  IsKnownFeatureProto(file->dependency(i)->name())) {
+                // Strip feature imports for editions codegen tests.
+                continue;
+              }
               thunks_printer->Emit(
                   {{"proto_dep_h", GetHeaderFile(ctx, *file->dependency(i))}},
                   R"cc(
@@ -218,9 +225,9 @@ bool RustGenerator::Generate(const FileDescriptor* file,
 #include "google/protobuf/map.h"
 #include "google/protobuf/repeated_field.h"
 #include "google/protobuf/repeated_ptr_field.h"
-#include "google/protobuf/rust/cpp_kernel/map.h"
-#include "google/protobuf/rust/cpp_kernel/serialized_data.h"
-#include "google/protobuf/rust/cpp_kernel/strings.h"
+#include "rust/cpp_kernel/map.h"
+#include "rust/cpp_kernel/serialized_data.h"
+#include "rust/cpp_kernel/strings.h"
         )cc");
   }
 
@@ -252,7 +259,6 @@ bool RustGenerator::Generate(const FileDescriptor* file,
       thunks_ctx.Emit({{"enum", enum_.full_name()}}, R"cc(
         // $enum$
       )cc");
-      GenerateEnumThunksCc(thunks_ctx, enum_);
       thunks_ctx.printer().PrintRaw("\n");
     }
   }
