@@ -18,9 +18,12 @@
 #include <cstdint>
 #include <type_traits>
 
+#include "absl/log/absl_check.h"
 #include "absl/types/span.h"
+#include "google/protobuf/arena.h"
 #include "google/protobuf/message_lite.h"
 #include "google/protobuf/parse_context.h"
+#include "google/protobuf/port.h"
 
 // Must come last:
 #include "google/protobuf/port_def.inc"
@@ -127,7 +130,7 @@ struct TcFieldData {
 struct TcParseTableBase;
 
 // TailCallParseFunc is the function pointer type used in the tailcall table.
-typedef const char* (*TailCallParseFunc)(PROTOBUF_TC_PARAM_DECL);
+typedef PROTOBUF_CC const char* (*TailCallParseFunc)(PROTOBUF_TC_PARAM_DECL);
 
 namespace field_layout {
 struct Offset {
@@ -272,6 +275,10 @@ struct alignas(uint64_t) TcParseTableBase {
   uint16_t extension_offset;
   uint32_t max_field_number;
   uint8_t fast_idx_mask;
+  // Testing one bit is cheaper than testing whether post_loop_handler is null,
+  // and we expect it to be null most of the time so no reason to load the
+  // pointer.
+  uint8_t has_post_loop_handler : 1;
   uint16_t lookup_table_offset;
   uint32_t skipmap32;
   uint32_t field_entries_offset;
@@ -280,7 +287,10 @@ struct alignas(uint64_t) TcParseTableBase {
   uint16_t num_aux_entries;
   uint32_t aux_offset;
 
-  const MessageLite* default_instance;
+  const ClassData* class_data;
+  using PostLoopHandler = const char* (*)(MessageLite* msg, const char* ptr,
+                                          ParseContext* ctx);
+  PostLoopHandler post_loop_handler;
 
   // Handler for fields which are not handled by table dispatch.
   TailCallParseFunc fallback;
@@ -302,7 +312,8 @@ struct alignas(uint64_t) TcParseTableBase {
                              uint32_t field_entries_offset,
                              uint16_t num_field_entries,
                              uint16_t num_aux_entries, uint32_t aux_offset,
-                             const MessageLite* default_instance,
+                             const ClassData* class_data,
+                             PostLoopHandler post_loop_handler,
                              TailCallParseFunc fallback
 #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
                              ,
@@ -313,13 +324,15 @@ struct alignas(uint64_t) TcParseTableBase {
         extension_offset(extension_offset),
         max_field_number(max_field_number),
         fast_idx_mask(fast_idx_mask),
+        has_post_loop_handler(post_loop_handler != nullptr),
         lookup_table_offset(lookup_table_offset),
         skipmap32(skipmap32),
         field_entries_offset(field_entries_offset),
         num_field_entries(num_field_entries),
         num_aux_entries(num_aux_entries),
         aux_offset(aux_offset),
-        default_instance(default_instance),
+        class_data(class_data),
+        post_loop_handler(post_loop_handler),
         fallback(fallback)
 #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
         ,
@@ -415,8 +428,6 @@ struct alignas(uint64_t) TcParseTableBase {
         : message_default_p(msg) {}
     constexpr FieldAux(const TcParseTableBase* table) : table(table) {}
     constexpr FieldAux(MapAuxInfo map_info) : map_info(map_info) {}
-    constexpr FieldAux(void (*create_in_arena)(Arena*, void*))
-        : create_in_arena(create_in_arena) {}
     constexpr FieldAux(LazyEagerVerifyFnType verify_func)
         : verify_func(verify_func) {}
     struct {
@@ -428,7 +439,6 @@ struct alignas(uint64_t) TcParseTableBase {
     const uint32_t* enum_data;
     const TcParseTableBase* table;
     MapAuxInfo map_info;
-    void (*create_in_arena)(Arena*, void*);
     LazyEagerVerifyFnType verify_func;
 
     const MessageLite* message_default() const {
@@ -463,6 +473,8 @@ struct alignas(uint64_t) TcParseTableBase {
                                    aux_offset +
                                    num_aux_entries * sizeof(FieldAux));
   }
+
+  const MessageLite* default_instance() const { return class_data->prototype; }
 };
 
 #if defined(_MSC_VER) && !defined(_WIN64)
@@ -532,6 +544,40 @@ static_assert(offsetof(TcParseTable<1>, fast_entries) ==
                   sizeof(TcParseTableBase),
               "Table entries must be laid out after TcParseTableBase.");
 
+template <typename T,
+          PROTOBUF_CC const char* (*func)(T*, const char*, ParseContext*)>
+PROTOBUF_CC const char* StubParseImpl(PROTOBUF_TC_PARAM_DECL) {
+  return func(static_cast<T*>(msg), ptr, ctx);
+}
+
+template <typename T,
+          PROTOBUF_CC const char* (*func)(T*, const char*, ParseContext*)>
+constexpr TcParseTable<0> CreateStubTcParseTable(
+    const ClassData* class_data,
+    TcParseTableBase::PostLoopHandler post_loop_handler = nullptr) {
+  return {
+      {
+          0,                  // has_bits_offset
+          0,                  // extension_offset
+          0,                  // max_field_number
+          0,                  // fast_idx_mask
+          0,                  // lookup_table_offset
+          0,                  // skipmap32
+          0,                  // field_entries_offset
+          0,                  // num_field_entries
+          0,                  // num_aux_entries
+          0,                  // aux_offset
+          class_data,         //
+          post_loop_handler,  //
+          nullptr,            // fallback
+#ifdef PROTOBUF_PREFETCH_PARSE_TABLE
+          nullptr,  // to_prefetch
+#endif              // PROTOBUF_PREFETCH_PARSE_TABLE
+      },
+      {{{StubParseImpl<T, func>, {}}}},
+  };
+}
+
 }  // namespace internal
 }  // namespace protobuf
 }  // namespace google
