@@ -1,35 +1,14 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 #include "google/protobuf/reflection_tester.h"
 
+#include <array>
+#include <string>
+
 #include <gtest/gtest.h>
 #include "absl/container/flat_hash_map.h"
 #include "google/protobuf/map_field.h"
@@ -44,7 +23,7 @@ namespace protobuf {
 MapReflectionTester::MapReflectionTester(const Descriptor* base_descriptor)
     : base_descriptor_(base_descriptor) {
   const DescriptorPool* pool = base_descriptor->file()->pool();
-  std::string package = base_descriptor->file()->package();
+  const absl::string_view package = base_descriptor->file()->package();
 
   map_enum_foo_ =
       pool->FindEnumValueByName(absl::StrCat(package, ".MAP_ENUM_FOO"));
@@ -206,6 +185,10 @@ MapReflectionTester::MapReflectionTester(const Descriptor* base_descriptor)
       EXPECT_EQ(fdesc->containing_type()->map_value(), fdesc);
     }
   }
+
+  // Must be heap allocated.
+  EXPECT_NE(long_string().capacity(), std::string().capacity());
+  EXPECT_NE(long_string_2().capacity(), std::string().capacity());
 }
 
 // Shorthand to get a FieldDescriptor for a field of unittest::TestMap.
@@ -294,14 +277,14 @@ void MapReflectionTester::SetMapFieldsViaReflection(Message* message) {
 
   sub_message = reflection->AddMessage(message, F("map_string_string"));
   sub_message->GetReflection()->SetString(sub_message, map_string_string_key_,
-                                          "0");
+                                          long_string());
   sub_message->GetReflection()->SetString(sub_message, map_string_string_val_,
-                                          "0");
+                                          long_string());
 
   sub_message = reflection->AddMessage(message, F("map_int32_bytes"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_bytes_key_, 0);
   sub_message->GetReflection()->SetString(sub_message, map_int32_bytes_val_,
-                                          "0");
+                                          long_string());
 
   sub_message = reflection->AddMessage(message, F("map_int32_enum"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_enum_key_, 0);
@@ -389,14 +372,14 @@ void MapReflectionTester::SetMapFieldsViaReflection(Message* message) {
 
   sub_message = reflection->AddMessage(message, F("map_string_string"));
   sub_message->GetReflection()->SetString(sub_message, map_string_string_key_,
-                                          "1");
+                                          long_string_2());
   sub_message->GetReflection()->SetString(sub_message, map_string_string_val_,
-                                          "1");
+                                          long_string_2());
 
   sub_message = reflection->AddMessage(message, F("map_int32_bytes"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_bytes_key_, 1);
   sub_message->GetReflection()->SetString(sub_message, map_int32_bytes_val_,
-                                          "1");
+                                          long_string_2());
 
   sub_message = reflection->AddMessage(message, F("map_int32_enum"));
   sub_message->GetReflection()->SetInt32(sub_message, map_int32_enum_key_, 1);
@@ -420,6 +403,7 @@ void MapReflectionTester::SetMapFieldsViaMapReflection(Message* message) {
   MapValueConstRef map_val_const;
 
   // Add first element.
+  std::string map_key_string;
   MapKey map_key;
   map_key.SetInt32Value(0);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_int32_int32"),
@@ -494,19 +478,20 @@ void MapReflectionTester::SetMapFieldsViaMapReflection(Message* message) {
                                                  map_key, &map_val));
   map_val.SetBoolValue(false);
 
-  map_key.SetStringValue("0");
+  map_key_string = long_string();
+  map_key.SetStringValue(map_key_string);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_string_string"),
                                           map_key, &map_val_const));
   EXPECT_TRUE(reflection->InsertOrLookupMapValue(
       message, F("map_string_string"), map_key, &map_val));
-  map_val.SetStringValue("0");
+  map_val.SetStringValue(long_string());
 
   map_key.SetInt32Value(0);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_int32_bytes"),
                                           map_key, &map_val_const));
   EXPECT_TRUE(reflection->InsertOrLookupMapValue(message, F("map_int32_bytes"),
                                                  map_key, &map_val));
-  map_val.SetStringValue("0");
+  map_val.SetStringValue(long_string());
 
   map_key.SetInt32Value(0);
   EXPECT_FALSE(reflection->LookupMapValue(*message, F("map_int32_enum"),
@@ -594,15 +579,16 @@ void MapReflectionTester::SetMapFieldsViaMapReflection(Message* message) {
                                      &map_val);
   map_val.SetBoolValue(true);
 
-  map_key.SetStringValue("1");
+  map_key_string = long_string_2();
+  map_key.SetStringValue(map_key_string);
   reflection->InsertOrLookupMapValue(message, F("map_string_string"), map_key,
                                      &map_val);
-  map_val.SetStringValue("1");
+  map_val.SetStringValue(long_string_2());
 
   map_key.SetInt32Value(1);
   reflection->InsertOrLookupMapValue(message, F("map_int32_bytes"), map_key,
                                      &map_val);
-  map_val.SetStringValue("1");
+  map_val.SetStringValue(long_string_2());
 
   map_key.SetInt32Value(1);
   reflection->InsertOrLookupMapValue(message, F("map_int32_enum"), map_key,
@@ -677,6 +663,7 @@ void MapReflectionTester::ModifyMapFieldsViaReflection(Message* message) {
   Message* sub_foreign_message;
 
   // Modify the second element
+  std::string map_key_string;
   MapKey map_key;
   map_key.SetInt32Value(1);
   EXPECT_FALSE(reflection->InsertOrLookupMapValue(message, F("map_int32_int32"),
@@ -743,7 +730,8 @@ void MapReflectionTester::ModifyMapFieldsViaReflection(Message* message) {
                                      &map_val);
   map_val.SetBoolValue(false);
 
-  map_key.SetStringValue("1");
+  map_key_string = long_string_2();
+  map_key.SetStringValue(map_key_string);
   reflection->InsertOrLookupMapValue(message, F("map_string_string"), map_key,
                                      &map_val);
   map_val.SetStringValue("2");
@@ -1231,9 +1219,9 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
     }
   }
   {
-    absl::flat_hash_map<bool, bool> map;
-    map[false] = false;
-    map[true] = true;
+    std::array<bool, 2> map;
+    map[0] = false;
+    map[1] = true;
     std::vector<bool> keys = {false, true};
     std::vector<bool> vals = {false, true};
     for (int i = 0; i < 2; i++) {
@@ -1248,7 +1236,7 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
                                                          map_bool_bool_key_);
         bool val = sub_message->GetReflection()->GetBool(*sub_message,
                                                          map_bool_bool_val_);
-        EXPECT_EQ(map[key], val);
+        EXPECT_EQ(map[key ? 1 : 0], val);
       } else {
         // Check with Map Reflection
         map_key.SetBoolValue(keys[i]);
@@ -1262,10 +1250,10 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
   }
   {
     absl::flat_hash_map<std::string, std::string> map;
-    map["0"] = "0";
-    map["1"] = "1";
-    std::vector<std::string> keys = {"0", "1"};
-    std::vector<std::string> vals = {"0", "1"};
+    map[long_string()] = long_string();
+    map[long_string_2()] = long_string_2();
+    std::vector<std::string> keys = {long_string(), long_string_2()};
+    std::vector<std::string> vals = {long_string(), long_string_2()};
     for (int i = 0; i < 2; i++) {
       const internal::MapFieldBase& map_field =
           reflection->GetRaw<internal::MapFieldBase>(message,
@@ -1292,8 +1280,8 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflection(
   }
   {
     absl::flat_hash_map<int32_t, std::string> map;
-    map[0] = "0";
-    map[1] = "1";
+    map[0] = long_string();
+    map[1] = long_string_2();
     for (int i = 0; i < 2; i++) {
       const internal::MapFieldBase& map_field =
           reflection->GetRaw<internal::MapFieldBase>(message,
@@ -1538,19 +1526,19 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
     }
   }
   {
-    absl::flat_hash_map<bool, bool> map;
-    map[false] = false;
-    map[true] = true;
+    std::array<bool, 2> map;
+    map[0] = false;
+    map[1] = true;
     for (MapIterator iter = reflection->MapBegin(message, F("map_bool_bool"));
          iter != reflection->MapEnd(message, F("map_bool_bool")); ++iter) {
-      EXPECT_EQ(map[iter.GetKey().GetBoolValue()],
+      EXPECT_EQ(map[iter.GetKey().GetBoolValue() ? 1 : 0],
                 iter.GetValueRef().GetBoolValue());
     }
   }
   {
     absl::flat_hash_map<std::string, std::string> map;
-    map["0"] = "0";
-    map["1"] = "1";
+    map[long_string()] = long_string();
+    map[long_string_2()] = long_string_2();
     int size = 0;
     for (MapIterator iter =
              reflection->MapBegin(message, F("map_string_string"));
@@ -1569,8 +1557,8 @@ void MapReflectionTester::ExpectMapFieldsSetViaReflectionIterator(
   }
   {
     absl::flat_hash_map<int32_t, std::string> map;
-    map[0] = "0";
-    map[1] = "1";
+    map[0] = long_string();
+    map[1] = long_string_2();
     for (MapIterator iter = reflection->MapBegin(message, F("map_int32_bytes"));
          iter != reflection->MapEnd(message, F("map_int32_bytes")); ++iter) {
       EXPECT_EQ(map[iter.GetKey().GetInt32Value()],
